@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2003 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org> for more information.
+ *  See <URL:http://www.coin3d.org/> for more information.
  *
  *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
- *  <URL:http://www.sim.no>.
+ *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
@@ -30,6 +30,9 @@
 // FIXME: share the relevant parts of the mainpage-doc with the other
 // So* libraries. (Doxygen supports conditional inclusion / exclusion
 // of doc parts.) 20020806 mortene.
+//
+// UPDATE 20030509 mortene: the mainpage documentation below is now
+// duplicated for SoWin.
 
 /*!
   \mainpage
@@ -62,7 +65,7 @@
   control the 3D-scene camera interaction.
 
   For a small, completely stand-alone usage example on how to
-  initialize the library and set up an viewer instance window, see the
+  initialize the library and set up a viewer instance window, see the
   following code:
 
   \code
@@ -176,7 +179,7 @@
   components (by the SoQtFullViewer).</li>
 
   <li>A set of high-level viewer classes, as has been presented by the
-  SoQtExaminerViewer in the above sourcecode example. There are
+  SoQtExaminerViewer in the above source code example. There are
   currently three different non-abstract viewer classes to choose
   from: the SoQtExaminerViewer (a plain model viewer), the
   SoQtFlyViewer (for fly-throughs in larger 3D scenes) and the
@@ -221,19 +224,13 @@
 // *************************************************************************
 
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
 #include <stdlib.h>
 #include <limits.h>
 
-/* The setting of this define needs to be added manually to
-   configure.in for all relevant projects. */
-#ifdef HAVE_X11_AVAILABLE
-#include <Inventor/Qt/devices/spwinput.h>
-#endif // HAVE_X11_AVAILABLE
-
 #include <qmainwindow.h>
 #include <qmessagebox.h>
 #include <qtimer.h>
@@ -241,8 +238,9 @@
 #include <qapplication.h>
 #include <qmetaobject.h>
 
-#include <Inventor/Qt/moc_SoQtP.icc>
-#include <Inventor/Qt/SoQtP.h>
+#ifdef Q_WS_X11
+#include <X11/Xlib.h>
+#endif // Q_WS_X11
 
 #include <Inventor/SoDB.h>
 #include <Inventor/SoInteraction.h>
@@ -250,17 +248,34 @@
 #include <Inventor/SbTime.h>
 #include <Inventor/errors/SoDebugError.h>
 
-#include <soqtdefs.h>
+#include <Inventor/Qt/SoQtP.h>
+#include <Inventor/Qt/moc_SoQtP.icc>
+/*[ src\Inventor\Qt\SoQtP.h | src\Inventor\Qt\moc_SoQtP.icc | $(QTDIR)\bin\moc -i @SOURCE@ -o @TARGET@ ]*/
+
 #include <Inventor/Qt/SoQt.h>
 #include <Inventor/Qt/SoQtObject.h>
 #include <Inventor/Qt/devices/SoQtDevice.h>
+/* The setting of this define needs to be added manually to
+   configure.in for all relevant projects. */
+#ifdef HAVE_X11_AVAILABLE
+#include <Inventor/Qt/devices/spwinput.h>
+#include <X11/Xlib.h>
+#endif // HAVE_X11_AVAILABLE
+
+#ifdef HAVE_WIN32_API
+#include <windows.h>
+#include <Inventor/Qt/devices/spwinput_win32.h>
+#endif // HAVE_WIN32_API
+
+
 #include <Inventor/Qt/SoQtComponent.h>
 #include <Inventor/Qt/SoAny.h>
 #include <Inventor/Qt/SoQtInternal.h>
 
-#ifdef Q_WS_X11
-#include <X11/Xlib.h>
-#endif // Q_WS_X11
+#include <Inventor/Qt/nodes/SoGuiNodes.h>
+#include <Inventor/Qt/engines/SoGuiEngines.h>
+
+#include <soqtdefs.h>
 
 
 // *************************************************************************
@@ -271,19 +286,49 @@
 
 QWidget * SoQtP::mainwidget = NULL;
 QApplication * SoQtP::appobject = NULL;
+SbBool SoQtP::madeappobject = FALSE;
 QTimer * SoQtP::idletimer = NULL;
 QTimer * SoQtP::timerqueuetimer = NULL;
 QTimer * SoQtP::delaytimeouttimer = NULL;
 SoQtP * SoQtP::slotobj = NULL;
 bool SoQtP::didcreatemainwidget = FALSE;
 
+#ifdef HAVE_WIN32_API
+// Qt 3.3 seems to use simple timers by default. Since simple timers
+// causes idle sensors to trigger only 100 times per second, we set
+// this variable to false in SoQt::init(). pederb, 2004-07-21
+
+// The global qt_win_use_simple_timers variable is defined in Qt 3.0.5
+// and 3.3.1. If linking fails for some version of Qt, we'll add a
+// configure test to test for this variable. Reading the Qt interest
+// archive, it seems like it's ok for application code to modify
+// it though.
+Q_EXPORT extern bool qt_win_use_simple_timers;
+#endif // HAVE_WIN32_API
+
+static void soqt_reset_simple_timers(void)
+{
+#ifdef HAVE_WIN32_API
+  const char * env = SoAny::si()->getenv("SOQT_USE_SIMPLE_TIMERS");
+  if (env) {
+    qt_win_use_simple_timers = (bool) atoi(env);
+  }
+  else {
+    qt_win_use_simple_timers = false;
+  }
+#endif // HAVE_WIN32_API
+}
+
 #define ENVVAR_NOT_INITED INT_MAX
 
 int SoQtP::DEBUG_X11SYNC = ENVVAR_NOT_INITED;
+const char * SoQtP::SOQT_XSYNC = "SOQT_XSYNC";
+int SoQtP::BRIL_X11_SILENCER = ENVVAR_NOT_INITED;
+int SoQtP::X11_ERRORHANDLER = ENVVAR_NOT_INITED;
+const char * SoQtP::SOQT_X11_ERRORHANDLER = "SOQT_X11_ERRORHANDLER";
 SoQtP_XErrorHandler * SoQtP::previous_handler = NULL;
 
 int SoQtP::DEBUG_LISTMODULES = ENVVAR_NOT_INITED;
-int SoQtP::BRIL_X11_SILENCER = ENVVAR_NOT_INITED;
 
 // We're using the singleton pattern to create a single SoQtP object
 // instance (a dynamic object is needed for attaching slots to signals
@@ -299,7 +344,7 @@ SoQtP::soqt_instance(void)
 int
 SoQtP::X11Errorhandler(void * d, void * ee)
 {
-#ifdef Q_WS_X11
+#if defined(Q_WS_X11)
 
   // This is a hack provided for one of our Coin PEL holders, to
   // silence X11 error output from some erroneous Qt code.
@@ -330,20 +375,36 @@ SoQtP::X11Errorhandler(void * d, void * ee)
 
   // Then the instructions:
 
-  SoDebugError::post("SoQtP::X11Errorhandler",
-                     "Detected probable Qt bug (or internal SoQt bug). %s %s",
-
-                     SoQtP::DEBUG_X11SYNC == 1 ? "" :
-                     "Set environment variable SOQT_XSYNC to \"1\" and "
-                     "re-run the application in a debugger with a breakpoint "
-                     "set on SoQtP::X11Errorhandler() to get a valid "
-                     "backtrace. "
-
+  SbString instructions = "";
+  if (! SoQtP::DEBUG_X11SYNC) {
+#ifdef __COIN__
+    instructions.sprintf("Set environment variable %s to \"1\" and "
+                         "re-run the application in a debugger with a "
+                         "breakpoint set on SoQtP::X11Errorhandler() to get a "
+                         "valid backtrace. "
                      "Then please forward the following information in an "
                      "e-mail to <coin-bugs@coin3d.org> along with the "
                      "backtrace. ",
+                         SoQtP::SOQT_XSYNC);
+#else // __COIN__
+    instructions = "Set environment variable SOQT_XSYNC to \"1\" and "
+                         "re-run the application in a debugger with a "
+                         "breakpoint set on SoQtP::X11Errorhandler() to get a "
+                         "valid backtrace. "
+                     "Then please forward the following information in an "
+                     "e-mail to <coin-bugs@coin3d.org> along with the "
+                     "backtrace. ";
+#endif // ! __COIN__
+  }
 
-                     depthsstr.getString());
+  SoDebugError::post("SoQtP::X11Errorhandler",
+                     "Detected probable Qt bug (or internal SoQt bug). %s %s",
+                     instructions.getString(), depthsstr.getString());
+
+  SoDebugError::post("SoQtP::X11Errorhandler",
+                     "If you don't want SoQt to catch X11 errors, set the %s "
+                     "environment variable to \"0\".",
+                     SoQtP::SOQT_X11_ERRORHANDLER);
 
   SoQtP::previous_handler(d, ee);
 #endif // Q_WS_X11
@@ -377,7 +438,7 @@ SoQtP::slot_timedOutSensor()
 void
 SoQtP::slot_idleSensor()
 {
-#if 0 // Disabled. hasPendingEvents() was probably not introduced until Qt 3
+#if defined(HAVE_QAPPLICATION_HASPENDINGEVENTS) && 0 // disabled, too much debug
   if (SOQT_DEBUG) { // debug
     SoDebugError::postInfo("SoQt::idleSensor",
                            "processing delay queue, "
@@ -385,13 +446,10 @@ SoQtP::slot_idleSensor()
                            SoQtP::appobject->hasPendingEvents());
     SoDebugError::postInfo("SoQt::idleSensor", "is %s",
                            SoQtP::idletimer->isActive() ? "active" : "inactive");
-  }
-#endif // disabled
+  } 
+#endif // HAVE_QAPPLICATION_HASPENDINGEVENTS
 
-  // FIXME: tmp disabled, as I'm not sure if this really
-  // helps. Investigate further the actual semantics of "idle trigger
-  // timers" in Qt. 20020910 mortene.
-#if 0
+#ifdef HAVE_QAPPLICATION_HASPENDINGEVENTS  
   // If there are still events in the Qt event queue, push the
   // idle-timer at the end of the queue again.
   if (SoQtP::appobject->hasPendingEvents()) {
@@ -399,7 +457,7 @@ SoQtP::slot_idleSensor()
     SoQtP::idletimer->start(0, TRUE);
     return;
   }
-#endif
+#endif // HAVE_QAPPLICATION_HASPENDINGEVENTS
 
   SoDB::getSensorManager()->processTimerQueue();
   SoDB::getSensorManager()->processDelayQueue(TRUE);
@@ -477,9 +535,13 @@ SoGuiP::sensorQueueChanged(void *)
   if (sm->isTimerSensorPending(t)) {
     SbTime interval = t - SbTime::getTimeOfDay();
     // Qt v2.1.1 (at least) on MSWindows will fail to trigger the
-    // timer if the interval is < 0.0. (And 0 has special meaning:
-    // trigger only when the application is idle and event queue is
-    // empty).
+    // timer if the interval is < 0.0.
+    //
+    // We also want to avoid setting it to 0.0, as that has a special
+    // semantic meaning: trigger only when the application is idle and
+    // event queue is empty -- which is not what we want to do here.
+    //
+    // So we clamp it, to a small positive value:
     if (interval.getValue() <= 0.0) { interval.setValue(1.0/5000.0); }
 
     if (SOQT_DEBUG && 0) { // debug
@@ -515,8 +577,11 @@ SoGuiP::sensorQueueChanged(void *)
     if (!SoQtP::idletimer->isActive()) SoQtP::idletimer->start(0, TRUE);
 
     if (!SoQtP::delaytimeouttimer->isActive()) {
-      unsigned long timeout = SoDB::getDelaySensorTimeout().getMsecValue();
-      SoQtP::delaytimeouttimer->start((int)timeout, TRUE);
+      const SbTime & t = SoDB::getDelaySensorTimeout();
+      if (t != SbTime::zero()) {
+        unsigned long timeout = t.getMsecValue();
+        SoQtP::delaytimeouttimer->start((int)timeout, TRUE);
+      }
     }
   }
   else {
@@ -532,12 +597,15 @@ SoGuiP::sensorQueueChanged(void *)
 // spaceball devices.)
 class SoQtApplication : public QApplication {
 public:
-  SoQtApplication(int argc, char ** argv) : QApplication(argc, argv) { }
+  SoQtApplication(int argc, char ** argv) : QApplication(argc, argv) { 
+#ifdef HAVE_X11_AVAILABLE
+    this->display = NULL;
+#endif // HAVE_X11_AVAILABLE
+  }
 #ifdef HAVE_X11_AVAILABLE
   virtual bool x11EventFilter(XEvent * e) {
     SPW_InputEvent sbEvent;
-    QWidget * topw = SoQt::getTopLevelWidget();
-    if (topw && SPW_TranslateEventX11(topw->x11Display(), e, &sbEvent)) {
+    if (SPW_TranslateEventX11(this->getDisplay(), e, &sbEvent)) {
       QWidget * focus = this->focusWidget();
       if (!focus) focus = this->activeWindow();
       if (focus) {
@@ -548,7 +616,46 @@ public:
     }
     return QApplication::x11EventFilter(e);
   }
+  Display * getDisplay(void) {
+    QWidget * topw = SoQt::getTopLevelWidget();
+    if (topw) return topw->x11Display();
+    if (this->display == NULL) {
+      // Keep a single display-ptr.
+      // 
+      // This resource is never deallocated explicitly (but of course
+      // implicitly by the system on application close-down). This to
+      // work around some strange problems with the NVidia-driver 29.60
+      // on XFree86 v4 when using XCloseDisplay() -- like doublebuffered
+      // visuals not working correctly.
+      //
+      // Also, XCloseDisplay() has been known to cause crashes when
+      // running remotely from some old Mesa version on Red Hat Linux
+      // 6.2 onto an IRIX6.5 display. It seems likely that this was
+      // caused by a bug in that particular old Mesa version.
+      //
+      // mortene@sim.no
+      this->display = XOpenDisplay(NULL);
+    }
+    return this->display;
+  }
+  Display * display;
 #endif // HAVE_X11_AVAILABLE
+
+#ifdef HAVE_WIN32_API
+  virtual bool winEventFilter(MSG * msg) {
+    SPW_InputEvent sbEvent;
+    if (SPW_TranslateEventWin32(msg, &sbEvent)) {
+      QWidget * focus = this->focusWidget();
+      if (!focus) focus = this->activeWindow();
+      if (focus) {
+        QCustomEvent qevent((QEvent::Type)SoQtInternal::SPACEBALL_EVENT,
+                            (void *)&sbEvent);
+        QApplication::sendEvent(focus, &qevent);
+      }
+    }
+    return QApplication::winEventFilter(msg);
+  }
+#endif // HAVE_WIN32_API
 };
 
 #endif // DOXYGEN_SKIP_THIS
@@ -570,8 +677,28 @@ COIN_IV_EXTENSIONS
 void
 SoQt::init(QWidget * toplevelwidget)
 {
-  assert(qApp && "you must set up a QApplication instance");
-  SoQtP::appobject = qApp;
+  // workaround for Qt bug under Windows. See above for comments
+  soqt_reset_simple_timers();
+
+  // "qApp" is a global variable from the Qt library, pointing to the
+  // single QApplication instance in a Qt-application.
+  if (qApp == NULL) {
+    // Set up the QApplication instance which we have derived into a
+    // subclass to catch spaceball events.
+
+    // use a static char array to store the dummy argv parameters
+    static char * dummyargv[1];
+    dummyargv[0] = "SoQt";
+    SoQtP::appobject = new SoQtApplication(1, (char **) dummyargv);
+    SoQtP::madeappobject = TRUE;
+  }
+  else {
+    // The user already set one up for us.
+    //
+    // FIXME: should somehow warn about the fact that spaceball events
+    // will not be caught, if a spaceball is attempted used. 20020619 mortene.
+    SoQtP::appobject = qApp;
+  }
 
   // This init()-method is called by the other 2 init()'s, so place
   // common code here.
@@ -595,22 +722,37 @@ SoQt::init(QWidget * toplevelwidget)
 #undef COIN_IV_EXTENSION
 #endif
 
+  SoGuiNodes::initClasses();
+  SoGuiEngines::initClasses();
+
   // This is _extremely_ useful for debugging X errors: activate this
   // code (set the SOQT_XSYNC environment variable on your system to
-  // "1"), then rerun the application code in a debugger with a
-  // breakpoint set at SoQtP::X11Errorhandler(). Now you can backtrace
-  // to the exact source location of the failing X request.
+  // "1") and SOQT_X11_ERRORHANDLER to "1", then rerun the application
+  // code in a debugger with a breakpoint set at
+  // SoQtP::X11Errorhandler(). Now you can backtrace to the exact
+  // source location of the failing X request.
+  //
+  // Note that we changed this from being opt-out to opt-in, due to
+  // what seemed like false negatives, and the fact that bad code from
+  // Qt would look like SoQt bugs.
 #ifdef Q_WS_X11
-  // Intervene upon X11 errors.
-  SoQtP::previous_handler = (SoQtP_XErrorHandler*)XSetErrorHandler((XErrorHandler)SoQtP::X11Errorhandler);
+  if (SoQtP::X11_ERRORHANDLER == ENVVAR_NOT_INITED) {
+    const char * env = SoAny::si()->getenv(SoQtP::SOQT_X11_ERRORHANDLER);
+    SoQtP::X11_ERRORHANDLER = env ? atoi(env) : 0;
+  }
+      
+  if (SoQtP::X11_ERRORHANDLER) {
+    // Intervene upon X11 errors.
+    SoQtP::previous_handler = (SoQtP_XErrorHandler*)XSetErrorHandler((XErrorHandler)SoQtP::X11Errorhandler);
 
-  if (SoQtP::DEBUG_X11SYNC == ENVVAR_NOT_INITED) {
-    const char * env = SoAny::si()->getenv("SOQT_XSYNC");
-    SoQtP::DEBUG_X11SYNC = env ? atoi(env) : 0;
-    if (SoQtP::DEBUG_X11SYNC) {
-      // FIXME: SoDebugError::initClass() not yet invoked! 20021021 mortene.
-      SoDebugError::postInfo("SoQt::init", "Turning on X synchronization.");
-      XSynchronize(qt_xdisplay(), True);
+    if (SoQtP::DEBUG_X11SYNC == ENVVAR_NOT_INITED) {
+      const char * env = SoAny::si()->getenv(SoQtP::SOQT_XSYNC);
+      SoQtP::DEBUG_X11SYNC = env ? atoi(env) : 0;
+      if (SoQtP::DEBUG_X11SYNC) {
+        // FIXME: SoDebugError::initClass() not yet invoked! 20021021 mortene.
+        SoDebugError::postInfo("SoQt::init", "Turning on X synchronization.");
+        XSynchronize(qt_xdisplay(), True);
+      }
     }
   }
 #endif // Q_WS_X11
@@ -634,8 +776,11 @@ SoQt::init(int & argc, char ** argv, const char * appname, const char * classnam
   // Must do this here so SoDebugError is initialized before it could
   // be attempted used.
   if (!SoDB::isInitialized()) { SoDB::init(); }
-  SoNodeKit::init();
-  SoInteraction::init();
+
+  // these aren't necessary - they are initialized in the SoQt::init method above
+  // which is invoked below
+  // SoNodeKit::init();
+  // SoInteraction::init();
 
   if (SOQT_DEBUG && (SoQtP::appobject || SoQtP::mainwidget)) {
     SoDebugError::postWarning("SoQt::init",
@@ -649,6 +794,7 @@ SoQt::init(int & argc, char ** argv, const char * appname, const char * classnam
     // Set up the QApplication instance which we have derived into a
     // subclass to catch spaceball events.
     SoQtP::appobject = new SoQtApplication(argc, argv);
+    SoQtP::madeappobject = TRUE;
   }
   else {
     // The user already set one up for us.
@@ -667,29 +813,7 @@ SoQt::init(int & argc, char ** argv, const char * appname, const char * classnam
   return SoQtP::mainwidget;
 }
 
-/*!
-  This is the event dispatch loop. It doesn't return until \a
-  QApplication::quit() or \a QApplication::exit() is called (which is
-  also done automatically by Qt whenever the user closes an
-  application's main widget).
-
-  An important note: be careful about how you handle
-  SoQtComponent-derived objects after the application control returns
-  from mainLoop(), as SoQt will then have been "cleaned up" with
-  regards to various internal resources. So doing for instance
-  something like this:
-
-  \code
-  SoQt::mainLoop();
-  viewer->hide();
-  \endcode
-
-  ..spells "undefined behavior, expect a crash".
-
-  \e Deleting a component after mainLoop() returns is allowed, though,
-  and also necessary to avoid getting reports of possible memory leaks
-  from memleak checkers.
-*/
+// documented in common/SoGuiCommon.cpp.in
 void
 SoQt::mainLoop(void)
 {
@@ -702,12 +826,7 @@ SoQt::mainLoop(void)
   SoQt::done();
 }
 
-/*!
-  This function will make the main event loop finish looping.
-
-  NOTE: exitMainLoop() is not present in the original API for SGI's
-  InventorXt library.
-*/
+// documented in common/SoGuiCommon.cpp.in
 void
 SoQt::exitMainLoop(void)
 {
@@ -735,26 +854,27 @@ SoQt::done(void)
     SoQtP::didcreatemainwidget = FALSE;
   }
 
-  // Notice that we *don't* delete the QApplication object, even
-  // though it originated with us, as it might be used someplace else
-  // in the application.
-  //
-  // (Keep this disabled code in place here, so we don't reactivate
-  // this by mistake again.)
-  //
-  // delete SoQtP::appobject; SoQtP::appobject = NULL;
-
   delete SoQtP::slotobj; SoQtP::slotobj = NULL;
+
+  // Notice that by default we *don't* delete the QApplication object,
+  // even though it originated with us, as it might be used someplace
+  // else in the application.
+  //
+  // Because there are situations where it is useful to have it
+  // deleted, we provide this possibility through setting an envvar
+  // (so the app programmer must actively seek out and know what she
+  // is doing):
+
+  if (SoQtP::madeappobject) {
+    const char * env = SoAny::si()->getenv("SOQT_DELETE_QAPPLICATION");
+    if (env && atoi(env) > 0) {
+      delete SoQtP::appobject; SoQtP::appobject = NULL;
+    }
+  }
 }
 
 
-/*!
-  Returns a pointer to the Qt QWidget which is the main widget for the
-  application. When this widget gets closed, SoQt::mainLoop() will
-  return (unless the close event is caught by the user).
-
-  \sa getShellWidget()
-*/
+// Documented in common/SoGuiCommon.cpp.in.
 QWidget *
 SoQt::getTopLevelWidget(void)
 {
@@ -774,12 +894,7 @@ SoQt::getTopLevelWidget(void)
 QWidget *
 SoQt::getShellWidget(const QWidget * w)
 {
-  if (SOQT_DEBUG && !w) {
-    SoDebugError::postWarning("SoQt::getShellWidget",
-                              "Called with NULL pointer.");
-    return NULL;
-  }
-
+  assert(w != NULL && "called with NULL pointer");
   return w->topLevelWidget();
 }
 
@@ -894,7 +1009,7 @@ SoQt::createSimpleErrorDialog(QWidget * widget,
     errstr += string2;
   }
 
-  QMessageBox::warning(widget, t.getString(), errstr.getString());
+  (void)QMessageBox::warning(widget, t.getString(), errstr.getString());
 }
 
 // *************************************************************************
