@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2003 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org> for more information.
+ *  See <URL:http://www.coin3d.org/> for more information.
  *
  *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
- *  <URL:http://www.sim.no>.
+ *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
@@ -25,25 +25,24 @@
 
 // *************************************************************************
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
 #include <qbitmap.h>
 #include <qwidget.h>
 #include <qmainwindow.h>
-#include <qmessagebox.h>
 #include <qapplication.h>
 #include <qmetaobject.h>
 #include <qcursor.h>
-#include <Inventor/Qt/moc_SoQtComponentP.icc>
 
 #include <Inventor/errors/SoDebugError.h>
 
-#include <soqtdefs.h>
+#include <Inventor/Qt/SoQtComponentP.h>
+#include <Inventor/Qt/moc_SoQtComponentP.icc>
+
 #include <Inventor/Qt/SoQt.h>
 #include <Inventor/Qt/SoQtComponent.h>
-#include <Inventor/Qt/SoQtComponentP.h>
 #include <Inventor/Qt/SoQtGLWidget.h>
 #include <Inventor/Qt/SoQtRenderArea.h>
 #include <Inventor/Qt/viewers/SoQtViewer.h>
@@ -54,11 +53,16 @@
 #include <Inventor/Qt/viewers/SoQtFlyViewer.h>
 #include <Inventor/Qt/SoAny.h>
 
+#include <soqtdefs.h>
+
 // debug
 #define SOQTCOMP_RESIZE_DEBUG 0
 
 static const char nullstring[] = "(null)";
 
+#define PRIVATE(obj) ((obj)->pimpl)
+#define PUBLIC(obj) ((obj)->pub)
+
 // *************************************************************************
 
 #ifndef DOXYGEN_SKIP_THIS // Skip internal class SoQtComponentP.
@@ -116,6 +120,10 @@ SoQtComponentP::getNativeCursor(const SoQtCursor::CustomCursor * cc)
   (void)memset(cursorbitmap, 0x00, MAXBITMAPBYTES);
   (void)memset(cursormask, 0x00, MAXBITMAPBYTES);
 
+  if ( !(cc->dim[0] <= MAXBITMAPWIDTH) )
+    printf("cursor bitmap width too large: %d\n", cc->dim[0]);
+  if ( !(cc->dim[1] <= MAXBITMAPHEIGHT) )
+    printf("cursor bitmap height too large: %d\n", cc->dim[1]);
   assert(cc->dim[0] <= MAXBITMAPWIDTH && "internal bitmap too large");
   assert(cc->dim[1] <= MAXBITMAPHEIGHT && "internal bitmap too large");
 
@@ -214,6 +222,15 @@ SoQtComponentP::eventFilter(QObject * obj, QEvent * e)
                          obj, eventnaming[e->type()]);
 #endif // debug
 
+  // The parent widget could receive resize events while we have still
+  // not completed the set up of our own widget, so we need to check
+  // for a NULL pointer.
+  //
+  // (One way to reproduce such a case: set up two SoQtExaminerViewer
+  // instances in an MDI workspace, first one, which is then
+  // maximized, then the second one instantiated will cause a crash.)
+  if (this->widget == NULL) { return FALSE; }
+
   if (e->type() == QEvent::Resize) {
     QResizeEvent * r = (QResizeEvent *)e;
 
@@ -516,6 +533,22 @@ SoQtComponent::show(void)
 
   PRIVATE(this)->widget->raise();
 
+  // FIXME: This is a very, very ugly workaround to force Qt 3.3.x on
+  // Mac OS X to repaint our component on startup. Otherwise, all you
+  // get is a blank white window. The problem seems to be related to a
+  // paint event "getting lost" somewhere (it's registered in
+  // SoQtComponent but never makes it to SoQtGLWidget.) This should
+  // really be fixed properly but I guess it's better to have things
+  // working in the meantime...
+  // Note that this workaround is a weird thing in itself: calling
+  // raise() twice should not make a difference, since raise() puts
+  // the widget to the top of the stack so that "the widget will be
+  // visually in front of any overlapping sibling widgets". 
+  // kyrah 20040211.
+#if (defined Q_WS_MAC && QT_VERSION >= 0x030300)
+  PRIVATE(this)->widget->raise();
+#endif // Q_WS_MAC
+
   if (SOQTCOMP_RESIZE_DEBUG) {  // debug
     SoDebugError::postInfo("SoQtComponent::show-4",
                            "raised %p: (%d, %d)",
@@ -617,8 +650,11 @@ void
 SoQtComponent::setIconTitle(const char * const title)
 {
   PRIVATE(this)->icontext = title;
-  if (this->getWidget())
-    SoQt::getShellWidget(this->getWidget())->setIconText(title);
+
+  QWidget * w = this->getWidget();
+  if (w && this->isTopLevelShell()) {
+    SoQt::getShellWidget(w)->setIconText(title);
+  }
 }
 
 // documented in common/SoGuiComponentCommon.cpp.in.
@@ -695,9 +731,10 @@ SoQtComponent::setSize(const SbVec2s size)
                          PRIVATE(this)->widget,
                          size[0], size[1]);
 #endif // debug
-  if (this->isTopLevelShell()) {
+  const SbBool yetbuilt = (this->getWidget() != NULL);
+  if (yetbuilt) {
     QWidget * shell = this->getShellWidget();
-    if (shell) shell->resize(size[0], size[1]);
+    if (shell) { shell->resize(size[0], size[1]); }
   }
   PRIVATE(this)->storesize = size;
   this->sizeChanged(size);
@@ -756,19 +793,24 @@ SoQtComponent::setFullScreen(const SbBool onoff)
   // as that means SoQtComponent::setFullScreen() will never work as
   // expected in the pre-compiled distro version we make.  20010608 mortene.
 
-#if HAVE_QWIDGET_SHOWFULLSCREEN
+  // prefer setWindowState() to showFullScreen() since
+  // setWindowState() will preserve other window flags/states.
+#if HAVE_QWIDGET_SETWINDOWSTATE
+  if (onoff) w->setWindowState(w->windowState() | Qt::WindowFullScreen);
+  else w->setWindowState(w->windowState() & ~Qt::WindowFullScreen);
+#elif HAVE_QWIDGET_SHOWFULLSCREEN
   if (onoff) w->showFullScreen();
   else w->showNormal();
-  PRIVATE(this)->fullscreen = onoff;
-  return TRUE;
-#else // !HAVE_QWIDGET_SHOWFULLSCREEN
+#else // HAVE_QWIDGET_SHOWFULLSCREEN
   SoDebugError::postWarning("SoQtComponent::setFullScreen",
                             "SoQt was compiled against version %s of Qt, "
                             "which doesn't have the "
                             "QWidget::showFullScreen() method",
                             QT_VERSION_STR);
   return FALSE;
-#endif // !HAVE_QWIDGET_SHOWFULLSCREEN
+#endif // !HAVE_QWIDGET_SHOWFULLSCREEN && !HAVE_QWIDGET_SETWINDOWSTATE 
+  PRIVATE(this)->fullscreen = onoff;
+  return TRUE;
 }
 
 // documented in common/SoGuiComponentCommon.cpp.in.
@@ -803,7 +845,7 @@ SoQtComponent::setWidgetCursor(QWidget * w, const SoQtCursor & cursor)
     // introduced another bug: when you click on the widget, the
     // cursor disappears. The Trolls have acknowledged that this is a
     // bug, and that it will be fixed in 3.1.3.
-#if defined Q_WS_MAC && ((QT_VERSION == 0x030100) || (QT_VERSION ==0x030101) || (QT_VERSION == 0x030102))
+#if defined Q_WS_MAC && ((QT_VERSION == 0x030100) || (QT_VERSION == 0x030101) || (QT_VERSION == 0x030102))
     w->setCursor(QCursor(Qt::arrowCursor));
     // spit out a warning that this is a Qt/Mac bug, not an SoQt problem
     static SbBool warningdisplayed = FALSE;
@@ -870,3 +912,7 @@ SoQtComponent::setWidgetCursor(QWidget * w, const SoQtCursor & cursor)
 }
 
 // *************************************************************************
+
+#undef PRIVATE
+#undef PUBLIC
+
