@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2003 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org> for more information.
+ *  See <URL:http://www.coin3d.org/> for more information.
  *
  *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
- *  <URL:http://www.sim.no>.
+ *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
@@ -54,28 +54,34 @@
 // throughout the compilation of the SoQt library, not just for this
 // file. 20021021 mortene.
 
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
+#include <stdlib.h>
+
 #define QT_CLEAN_NAMESPACE 1
 
 #include <qevent.h>
 #include <qframe.h>
-
 #include <qmetaobject.h>
+
+#include <Inventor/SbTime.h>
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/misc/SoBasic.h>
+
+#include <Inventor/Qt/SoQtGLWidgetP.h>
 #include <Inventor/Qt/moc_SoQtGLWidgetP.icc>
 
 #include <Inventor/Qt/SoAny.h>
 #include <Inventor/Qt/SoQtBasic.h>
 #include <Inventor/Qt/SoQtGLWidget.h>
-#include <Inventor/Qt/SoQtGLWidgetP.h>
 #include <Inventor/Qt/widgets/SoQtGLArea.h>
-#include <Inventor/SbTime.h>
-#include <Inventor/errors/SoDebugError.h>
-#include <Inventor/misc/SoBasic.h>
 #include <soqtdefs.h>
-#include <stdlib.h>
 
-#if HAVE_CONFIG_H
-#include <config.h>
-#endif // HAVE_CONFIG_H
+#define PRIVATE(obj) ((obj)->pimpl)
+#define PUBLIC(obj) ((obj)->pub)
 
 // Abstract the overlay handling methods on QGLFormat and QGLWidget,
 // because they may or may not be there depending on the version of
@@ -366,6 +372,28 @@ SoQtGLWidget::getStencilBuffer(void) const
   return (SbBool) (PRIVATE(this)->glformat->stencil());
 }
 
+// Documented in common/SoGuiGLWidgetCommon.cpp.in.
+void 
+SoQtGLWidget::setAlphaChannel(const SbBool enable)
+{
+  if ((enable && PRIVATE(this)->glformat->alpha()) ||
+      (!enable && !PRIVATE(this)->glformat->alpha()))
+    return;
+
+  PRIVATE(this)->glformat->setAlpha(enable);
+
+  // Rebuild if a GL widget has already been built.
+  if (PRIVATE(this)->currentglwidget) PRIVATE(this)->buildGLWidget();
+}
+
+// Documented in common/SoGuiGLWidgetCommon.cpp.in.
+SbBool 
+SoQtGLWidget::getAlphaChannel(void) const
+{
+  return (SbBool) (PRIVATE(this)->glformat->alpha());
+}
+
+
 
 // Documented in common/SoGuiGLWidgetCommon.cpp.in.
 void
@@ -426,22 +454,21 @@ SoQtGLWidget::setGLSize(const SbVec2s size)
     int frame = this->isBorder() ? PRIVATE(this)->borderthickness : 0;
     PRIVATE(this)->currentglwidget->setGeometry(QRect(frame, frame, PRIVATE(this)->glSize[0], PRIVATE(this)->glSize[1]));
   }
-  
+
 // Due to an internal hack in Qt/Mac 3.1.x, sometimes the OpenGL context
-// is destroyed and re-created (see
-// QGLWidget::macInternalRecreateContext()). 
+// is destroyed and re-created (see QGLWidget::macInternalRecreateContext()). 
 // In this case, we must register a new context as well in order to get
 // a new SoGLRenderAction cache context id.
 #if (defined Q_WS_MAC && QT_VERSION >= 0x030100) 
   QGLWidget * w = (QGLWidget*) this->getGLWidget();
-  if (w && PRIVATE(this)->oldcontext != w->context()) {
-    if (SOQT_DEBUG && 0) {
-      SoDebugError::postInfo("SoQtGLWidget::setGLSize",
+  if (w && PRIVATE(this)->oldcontext != w->context()) { 
+    if (SOQT_DEBUG && 0) { 
+      SoDebugError::postInfo("SoQtGLWidget::setGLSize", 
                              "OpenGL context recreated by Qt, "
                              "registering new context.");
     }
     SoAny::si()->unregisterGLContext((void *)this);
-    SoAny::si()->registerGLContext((void *)this, 0, 0);
+    SoAny::si()->registerGLContext((void *)this, 0, 0);  
   }
   if (w) PRIVATE(this)->oldcontext = w->context();
 #endif
@@ -800,12 +827,13 @@ SoQtGLWidgetP::eventFilter(QObject * obj, QEvent * e)
                            eventnaming[e->type()], e->type());
   }
 
+  QEvent::Type etype = e->type();
 #if QT_VERSION >= 200
   // Qt 2 introduced "accelerator" type keyboard events, which should
   // simply be ignored (all keyboard events are first attempted passed
   // by the Qt event engine as accelerator events, before they are
   // re-sent as "ordinary" keyboard events).
-  if (e->type() == QEvent::Accel || e->type() == QEvent::AccelAvailable) {
+  if (etype == QEvent::Accel || etype == QEvent::AccelAvailable) {
     ((QKeyEvent *)e)->ignore();
     // It might not matter whether we return TRUE or FALSE here, but
     // it seems more natural to return FALSE according to the
@@ -819,15 +847,20 @@ SoQtGLWidgetP::eventFilter(QObject * obj, QEvent * e)
   // mouse events were routed to the gl widget, even if it was
   // supposed to go somewhere else. I'm not sure if this is the
   // correct fix though. pederb, 2001-10-16
-  if ((e->type() == QEvent::MouseButtonPress ||
-       e->type() == QEvent::MouseButtonRelease ||
-       e->type() == QEvent::MouseButtonDblClick ||
-       e->type() == QEvent::MouseMove) &&
-      (obj != this->currentglwidget)) return FALSE;
 
-  SbBool keyboardevent =
-    (e->type() == QEvent::KeyPress) || (e->type() == QEvent::KeyRelease);
-  if (keyboardevent) {
+  if ((etype == QEvent::MouseButtonPress ||
+       etype == QEvent::MouseButtonRelease ||
+       etype == QEvent::MouseButtonDblClick ||
+       etype == QEvent::MouseMove) &&
+      (obj != this->currentglwidget)) {
+    return FALSE;
+  }
+
+  int kbdevent = FALSE;
+  if ( QEvent::KeyPress == etype ) kbdevent = TRUE;
+  if ( QEvent::KeyRelease == etype ) kbdevent = TRUE;
+
+  if (kbdevent) {
     // Ignore keyboard-events, as they are caught directly by the
     // SoQtGLArea widget and forwarded through the
     // SoQtGLWidgetP::GLAreaKeyEvent() callback.
@@ -987,10 +1020,80 @@ SoQtGLWidgetP::buildGLWidget(void)
     delete wasprevious;
   }
 
+  // FIXME: the panic below has been reported from a customer of dGB,
+  // on Windows, with a WildCat card and driver. Consider finding
+  // appropriate workarounds. Hard task, but needs to be done, as this
+  // is a *really* ugly problem.
+  //
+  // Snippets from my mail to dGB about the problem:
+  //
+  // -------8<------ [snip] -------------------8<------ [snip] ------------
+  // As for the cause of the problem: it is actually the QGLWidget class
+  // from Troll Tech's Qt library which fails to find a usable OpenGL
+  // context. We check for the QGLWidget::isValid() flag, and in your
+  // customer's case it returns FALSE. According to the API documentation
+  // of QGLWidget::isValid():
+  // 
+  //         [...] A widget will be invalid if the system has no OpenGL
+  //         support.
+  // 
+  // You said your customer know for sure other OpenGL programs work on his
+  // display, so this is obviously bogus.
+  // 
+  // I.e. it's technically not our bug, but Troll Tech's.
+  // 
+  // 
+  // I would like to tell you there is a quick workaround fix we could
+  // apply, but I'm afraid TT has made that difficult, because the OpenGL
+  // context selection is almost totally opaque -- we just have to work
+  // with what we get from Qt.
+  // 
+  // You could try to upgrade your Qt library version, but I doubt it would
+  // help, as I would guess that this is a quite obscure bug.
+  // 
+  // From our own experiences with low-level OpenGL/windowsystem handling
+  // with SoWin, I would say that it is probably triggered by QGLWidget's
+  // selection of a resource-intensive pixelformat for the OpenGL canvas
+  // (by using the Win32 API's ChoosePixelFormat()), which when attempted
+  // set up will actually find itself short on available resources.
+  // 
+  // Having then painted itself into a corner, the QGLWidget has not been
+  // written in a robust enough manner that it can deploy any fallback.
+  // -------8<------ [snip] -------------------8<------ [snip] ------------
+  //
+  // 20031218 mortene.
+  //
+  // UPDATE 20040624 mortene: some additional suggestions made to
+  // another SoQt user experiencing the same problems:
+  //
+  // -------8<------ [snip] -------------------8<------ [snip] ------------
+  // As mentioned in the FIXME at SoQtGLWidget.cpp:1014 [i.e. this
+  // FIXME], this is a tough problem, and it will probably take some
+  // hacking around inside the workings of Qt's QGLWidget to debug and
+  // solve properly.
+  // 
+  // The first thing to do is perhaps to upgrade Qt to the very latest
+  // version, and re-try with that. It could be that Troll Tech has
+  // fixed the problem in their later Qt versions.
+  // 
+  // Second, ask for assistance on their support channels.
+  // 
+  // If that doesn't help, perhaps make a stand-alone QGLWidget
+  // example (i.e.  with no SoQt or Coin), and try to get it
+  // reproduced with that. It will perhaps not be flawed in the
+  // default QGLFormat configuration, you might have to try to match
+  // exactly the features we ask for for the QGLWidget in SoQt.
+  // 
+  // With a stand-alone Qt example which demonstrates the bug, I guess
+  // it should be possible to a) find a work-around (e.g. by avoiding
+  // certain features for the QGLWidget?), and b) get assistance from
+  // TT's support personnel.
+  // -------8<------ [snip] -------------------8<------ [snip] ------------
+
   if (!this->currentglwidget->isValid()) {
     SbString s =
       "Can't set up a valid OpenGL canvas, "
-      "something is seriously wrong with your system!";
+      "something is seriously wrong with the system!";
     SbBool handled =
       SoAny::si()->invokeFatalErrorHandler(s, SoQt::NO_OPENGL_CANVAS);
     if (handled) { return; }
@@ -1114,3 +1217,7 @@ SoQtGLWidgetP::isDirectRendering(void)
 #endif // DOXYGEN_SKIP_THIS
 
 // *************************************************************************
+
+#undef PRIVATE
+#undef PUBLIC
+
