@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2003 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,14 +14,14 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org> for more information.
+ *  See <URL:http://www.coin3d.org/> for more information.
  *
  *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
- *  <URL:http://www.sim.no>.
+ *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
@@ -40,6 +40,10 @@
 // against Qt versions >= 2.2.0.
 #define QT_KEYPAD_MASK_ASSUMED 0x4000
 
+#if defined(Q_WS_MAC) || (defined(__APPLE__) && defined(Q_WS_X11))
+#define QT_ON_MAC 1
+#endif
+
 #if HAVE_QT_KEYPAD_DEFINE
 #define QT_KEYPAD_MASK Qt::Keypad
 #else // !HAVE_QT_KEYPAD_DEFINE
@@ -54,6 +58,9 @@
 
 // *************************************************************************
 
+#define PRIVATE(obj) ((obj)->pimpl)
+#define PUBLIC(obj) ((obj)->pub)
+
 #ifndef DOXYGEN_SKIP_THIS
 
 class SoQtKeyboardP : public SoGuiKeyboardP {
@@ -97,7 +104,7 @@ struct SoQtKeyboardP::key1map SoQtKeyboardP::QtToSoMapping[] = {
 
   {Qt::Key_Shift, SoKeyboardEvent::LEFT_SHIFT, '.'},
   {Qt::Key_Control, SoKeyboardEvent::LEFT_CONTROL, '.'},
-#if !(defined Q_WS_MAC && QT_VERSION >= 0x030100)
+#if !(defined QT_ON_MAC && QT_VERSION >= 0x030100)
 // Since Qt/Mac 3.1.x, the "ctrl" key is mapped to Qt::MetaButton (not
 // Qt::ControlButton) => We must not handle META as ALT on Mac OS X.
   {Qt::Key_Meta, SoKeyboardEvent::LEFT_ALT, '.'},
@@ -429,7 +436,7 @@ SoQtKeyboard::translateEvent(QEvent * event)
       case Qt::Key_Shift: state |= Qt::ShiftButton; break;
       case Qt::Key_Control: state |= Qt::ControlButton; break;
       case Qt::Key_Alt: state |= Qt::AltButton; break;
-#if !(defined Q_WS_MAC && QT_VERSION >= 0x030100)
+#if !(defined QT_ON_MAC && QT_VERSION >= 0x030100)
 // Since Qt/Mac 3.1.x, the "ctrl" key is mapped to Qt::MetaButton (not
 // Qt::ControlButton) => We must not handle META as ALT on Mac OS X.
       case Qt::Key_Meta: state |= Qt::AltButton; break;
@@ -471,3 +478,7 @@ SoQtKeyboard::translateEvent(QEvent * event)
 }
 
 // *************************************************************************
+
+#undef PRIVATE
+#undef PUBLIC
+
