@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2003 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org> for more information.
+ *  See <URL:http://www.coin3d.org/> for more information.
  *
  *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
- *  <URL:http://www.sim.no>.
+ *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
@@ -58,13 +58,13 @@
 #include <qstylefactory.h>
 #endif // HAVE_QSTYLEFACTORY_H
 
-#include <Inventor/Qt/viewers/moc_SoQtFullViewerP.icc>
-#include <Inventor/Qt/viewers/SoQtFullViewerP.h>
-
 #include <Inventor/nodes/SoPerspectiveCamera.h>
 #include <Inventor/nodes/SoOrthographicCamera.h>
 #include <Inventor/errors/SoDebugError.h>
 
+#include <Inventor/Qt/viewers/SoQtFullViewerP.h>
+#include <Inventor/Qt/viewers/moc_SoQtFullViewerP.icc>
+
 #include <soqtdefs.h>
 #include <Inventor/Qt/SoQt.h>
 #include <Inventor/Qt/widgets/SoQtThumbWheel.h>
@@ -561,6 +561,7 @@ SoQtFullViewer::buildViewerButtons(QWidget * parent)
   QWidget * w = new QWidget(parent);
   this->createViewerButtons(w, PRIVATE(this)->viewerbuttons);
   w->move(0, 0);
+  w->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
 
   // assert(PRIVATE(this)->viewerbuttons->getLength() != 0);
   QGridLayout * l =
@@ -728,6 +729,7 @@ SoQtFullViewer::setRightWheelString(const char * const string)
 void
 SoQtFullViewer::sizeChanged(const SbVec2s & size)
 {
+
 #if SOQT_DEBUG && 0
   SoDebugError::postInfo("SoQtFullViewer::sizeChanged", "(%d, %d)",
                          size[0], size[1]);
@@ -750,19 +752,19 @@ SoQtFullViewer::sizeChanged(const SbVec2s & size)
 #if defined Q_WS_MAC && ((QT_VERSION == 0x030100) || (QT_VERSION == 0x030101))
 
   // Environment variable to override Qt/Mac 3.1.x workarounds.
-  const char * forcenoresizeworkaround =
+  const char * forcenoresizeworkaround = 
     SoAny::si()->getenv("FORCE_NO_QTMAC_31_RESIZE_WORKAROUND");
   if (!forcenoresizeworkaround || (atoi(forcenoresizeworkaround) == 0)) {
 
     if (!PRIVATE(this)->decorations) { // GL context would fill whole window
-      newsize -= SbVec2s(0, 15);       // -> leave lowermost 15 px blank
+       newsize -= SbVec2s(0, 15);       // -> leave lowermost 15 px blank
 
-      // spit out a warning that this is a Qt/Mac bug, not an SoQt  problem
+      // spit out a warning that this is a Qt/Mac bug, not an SoQt problem
       const char * env = SoAny::si()->getenv("SOQT_NO_QTMAC_BUG_WARNINGS");
-      if (!env || !atoi(env)) {
-        SoDebugError::postWarning("SoQtFullViewer::sizeChanged",
+      if (!env || !atoi(env)) {        
+        SoDebugError::postWarning("SoQtFullViewer::sizeChanged", 
                                   "\nThis version of Qt/Mac contains a bug "
-                                  "that makes it necessary to leave the\n"
+                                  "that makes it necessary to leave the\n"  
                                   "lowermost 15 pixels of the viewer window "
                                   "blank. Set the environment variable\n"
                                   "FORCE_NO_QTMAC_31_RESIZE_WORKAROUND=1 to "
@@ -770,11 +772,11 @@ SoQtFullViewer::sizeChanged(const SbVec2s & size)
                                   "You can turn off warnings about Qt/Mac "
                                   "bugs permanently by setting \n"
                                   "SOQT_NO_QTMAC_BUG_WARNINGS=1.\n");
-      }
-    }
+      } 
+    } 
   }
 
-#endif
+#endif 
 
   newsize = SbVec2s(SoQtMax(newsize[0], (short)1),
                     SoQtMax(newsize[1], (short)1));
@@ -1001,3 +1003,7 @@ void SoQtFullViewerP::rightWheelReleased(void) { PUBLIC(this)->rightWheelFinish(
 // *************************************************************************
 
 #endif // DOXYGEN_SKIP_THIS
+
+#undef PUBLIC
+#undef PRIVATE
+
